@@ -4,7 +4,7 @@
 #'   evaluate covariate values at data points and at pixels
 #'   together with intensity of null/reference model
 #'
-#' $Revision: 1.55 $ $Date: 2025/06/02 07:00:33 $
+#' $Revision: 1.57 $ $Date: 2025/11/09 00:21:11 $
 #'
 
 ## Code for generic spatialCovariateEvidence() is moved to spatstat.explore
@@ -238,6 +238,7 @@ spatialCovariateEvidence.ppm <- local({
                    lambda      = lambda,
                    lambdaX     = lambdaX,
                    weights     = pixelarea,
+                   EdN         = lambda * pixelarea, # increment of lamba(u) du
                    ZX          = ZX,
                    type        = type)
     return(list(values=values, info=info, X=X)) # X is possibly a subset of original
@@ -267,6 +268,7 @@ spatialCovariateEvidence.ppm <- local({
   spatialCovariateEvidence.ppm
 })
 
+spatialCovariateEvidence.dppm <- 
 spatialCovariateEvidence.kppm <- function(model, covariate, ...) {
   mname <- singlestring(short.deparse(substitute(model)))
   cname <- singlestring(short.deparse(substitute(covariate)))
